@@ -9,8 +9,36 @@ from flask import Flask, render_template;
 
 #region UTILITY METHODS
 
-    # Replace first letter of a string with the same letter, in Uppercase
-    ## Used to make sure that lower case Class names, and lower case Function names, get Uppercased for the generated routes
+# Copy sharpify.js to static folder
+def use_htmlAttributes(staticFolderName:str = "static/js"):
+
+    """
+    Copies the sharpify.js file, to your static folder.
+    Allows the usage of "data-controller" & "data-action" attributes in ANCHOR & FORM elements, to route hrefs and actions.
+    - Relative path to your static folder, and whatever sub folder you are storing your .js files in.
+    - Path will be combined with the path to the module, where this function is being called from.
+    - Example usage: <script src="{{ url_for('static',filename='js/sharpify.js') }}"></script>
+    """
+
+    # Filter the input
+    if staticFolderName[0] == "/":
+        staticFolderName = staticFolderName.replace("/", "", 1);
+
+    fileContent = "";
+
+    with open(Path(__file__).parent.joinpath("sharpify.js"), "r") as file:
+        fileContent = file.read();
+    
+    fi = inspect.stack()[1];
+    mod = inspect.getmodule(fi[0]);
+    path = Path(mod.__file__).parent.joinpath(staticFolderName).joinpath("sharpify.js");
+    print(path);
+
+    with open(path, "w") as file:
+        file.write(fileContent);
+
+# Replace first letter of a string with the same letter, in Uppercase
+## Used to make sure that lower case Class names, and lower case Function names, get Uppercased for the generated routes
 def string_flUppercase(value: str):
     firstLetter = value[0];
     return value.replace(firstLetter, firstLetter.upper(), 1);
@@ -71,7 +99,7 @@ def getFunctionMethods(funcName:str, classDecorators:dict):
 
     return methodList;
 
-def map_routes(flask_app: Flask, default_controller: str, default_action: str):
+def use_mvc(flask_app: Flask, default_controller: str, default_action: str):
 
     """ 
     Definition:
@@ -121,39 +149,68 @@ def map_routes(flask_app: Flask, default_controller: str, default_action: str):
 
                         func_list = inspect.getmembers(attribute, inspect.isfunction)
                         for func in func_list:
+                            print("---" + func[0] + "----");
 
+                            controllerName = string_flUppercase(attribute.__name__);
                             funcName = string_flUppercase(func[0]);
                             methods = getFunctionMethods(funcName, classDecos);
+                            routeArguements = getFunctionArguements(func[1]);
                             print(methods);
 
-                            if attribute.__name__ == default_controller and func[0] == default_action:
-                                flask_app.add_url_rule("/", "default_" + funcName, func[1]);
+
+
+                            # Route State Checks
+                            isDefaultRoute = attribute.__name__ == default_controller and func[0] == default_action;
+                            isDefaultAction = func[0] == default_action;
+                            hasArguements = len(inspect.getfullargspec(func[1]).args) > 0;
+                            isOptionalArgs = inspect.getfullargspec(func[1]).defaults != None;
+                            print("IsDefaultRoute: " + str(isDefaultRoute));
+                            print("IsDefaultAction: " + str(isDefaultAction));
+                            print("HasArguements: " + str(hasArguements));
+                            print("IsOptionalArgs: " + str(isOptionalArgs));
+
+
+                            if isDefaultRoute:
+                                flask_app.add_url_rule("/", "default_route_" + funcName, func[1], methods=methods);
                                 print("GENERATE DEFAULT ROUTE: " + "/");
                                 print("MAPS TO: " + attribute.__name__.replace("Controller", "") + "/" + funcName + "/");
                             
 
+                            if isDefaultAction:
+                                if hasArguements:
+                                    rule = "/" + controllerName.replace("Controller", "") + routeArguements;
+                                    endpoint = attribute.__name__ + "_" + func[0] + "_" + "default_args";
+                                    flask_app.add_url_rule(rule, endpoint, func[1], methods=methods);
+                                    print("GENERATED DEFAULT ACTION ROUTE /w ARG: " + rule);
+                                    
+                                    if isOptionalArgs:
+                                        rule = "/" + controllerName.replace("Controller", "") + "/";
+                                        endpoint = attribute.__name__ + "_" + func[0] + "_" + "default_optional";
+                                        flask_app.add_url_rule(rule, endpoint, func[1], methods=methods);
+                                        print("GENERATED DEFAULT ACTION ROUTE /w OPTIONAL ARG: " + rule);
+                                else:
+                                    rule = "/" + controllerName.replace("Controller", "") + "/";
+                                    endpoint = attribute.__name__ + "_" + func[0] + "_" + "default";
+                                    flask_app.add_url_rule(rule, endpoint, func[1], methods=methods);
+                                    print("GENERATED DEFAULT ACTION ROUTE: " + rule);
 
-                            # GENERATE ROUTES
-                            controllerName = string_flUppercase(attribute.__name__);
+                            else:
+                                if hasArguements:
+                                    rule = "/" + controllerName.replace("Controller", "") + "/" + funcName + routeArguements;
+                                    endpoint = attribute.__name__ + "_" + func[0] + "_" + "args";
+                                    flask_app.add_url_rule(rule, endpoint, func[1], methods=methods);
+                                    print("GENERATED ROUTE /w ARG: " + rule);
 
-                            # Does the function contain an optional argument?
-                            fullArgSpec = inspect.getfullargspec(func[1]);
-                            if len(fullArgSpec.args) and fullArgSpec.defaults != None:
-                                rule = "/" + controllerName.replace("Controller", "") + "/" + funcName + "/";
-                                endpoint = attribute.__name__+ "_" + func[0] + "_" + "optional";
-                                view_func = func[1];
-                                flask_app.add_url_rule(rule, endpoint, view_func, methods=methods);
-                                print("GENERATED OPTIONAL ROUTE: " + rule);
-
-                            
-                            # Add route that will either be without arguement, or with an arguement
-                            routeArguements = getFunctionArguements(func[1]);
-                            rule = ("/" + controllerName.replace("Controller", "") + "/" + funcName + routeArguements);
-                            endpoint = attribute.__name__ + "_" + func[0];
-                            view_func = func[1];
-                            
-                            flask_app.add_url_rule(rule, endpoint, view_func, methods=methods);
-                            print("GENERATED ROUTE: " + rule);
+                                    if isOptionalArgs:
+                                        rule = "/" + controllerName.replace("Controller", "") + "/" + funcName;
+                                        endpoint = attribute.__name__ + "_" + func[0] + "_" + "optional";
+                                        flask_app.add_url_rule(rule, endpoint, func[1], methods=methods);
+                                        print("GENERATED ROUTE /w OPTIONAL ARG: " + rule);
+                                else:
+                                    rule = "/" + controllerName.replace("Controller", "") + "/" + funcName;
+                                    endpoint = attribute.__name__ + "_" + func[0] + "_" + "optional";
+                                    flask_app.add_url_rule(rule, endpoint, func[1], methods=methods);
+                                    print("GENERATED ROUTE: " + rule);
 
 #endregion
 
